@@ -14,8 +14,8 @@ namespace Tomboy
 	public class NoteTag : Gtk.TextTag
 	{
 		string element_name;
-		Gdk.Pixbuf image;
-		Gtk.TextMark imageLocation;
+		Gtk.TextMark widgetLocation;
+		Gtk.Widget widget;
 
 		[Flags]
 		enum TagFlags {
@@ -235,23 +235,52 @@ namespace Tomboy
 
 		public virtual Gdk.Pixbuf Image
 		{
-			get { return image; }
+			get {
+				Gtk.Image image = widget as Gtk.Image;
+				if (image == null) return null;
+
+				return image.Pixbuf;
+			}
 			set {
-				image = value;
+				if(value == null) {
+					Widget = null;
+					return;
+				}
+
+				Gtk.Image image = new Gtk.Image (value);
+				Widget = image;
+			}
+		}
+
+		public virtual Gtk.Widget Widget
+		{
+			get { return widget; }
+			set {
+				if (value == null && widget != null) {
+					widget.Destroy ();
+					widget = null;
+				}
+
+				widget = value;
 
 				if (Changed != null) {
 					Gtk.TagChangedArgs args = new Gtk.TagChangedArgs ();
+					args.Args = new object [2];
 					args.Args [0] = false; // SizeChanged
 					args.Args [1] = this;  // Tag
-					Changed (this, args);
+					try {
+						Changed (this, args);
+					} catch (Exception e) {
+						Logger.Warn ("Exception calling TagChanged from NoteTag.set_Widget: {0}", e.Message);
+					}
 				}
 			}
 		}
 
-		public virtual Gtk.TextMark ImageLocation
+		public virtual Gtk.TextMark WidgetLocation
 		{
-			get { return imageLocation; }
-			set { imageLocation = value; }
+			get { return widgetLocation; }
+			set { widgetLocation = value; }
 		}
 
 		public event Gtk.TagChangedHandler Changed;
@@ -315,17 +344,23 @@ namespace Tomboy
 	public class DepthNoteTag : NoteTag
 	{
 		int depth = -1;
+		Pango.Direction direction = Pango.Direction.Ltr;
 		
 		public int Depth
 		{
 			get{ return depth; }
-			set{ depth = value; }
+		}
+		
+		public Pango.Direction Direction
+		{
+			get{ return direction; }
 		}
 
-		public DepthNoteTag (int depth)
-			: base("depth:" + depth)
+		public DepthNoteTag (int depth, Pango.Direction direction)
+			: base("depth:" + depth + ":" + direction)
 		{
-			Depth = depth;
+			this.depth = depth;
+			this.direction = direction;
 		}
 
 		public override void Write (XmlTextWriter xml, bool start)
@@ -333,6 +368,14 @@ namespace Tomboy
 			if (CanSerialize) {
 				if (start) {
 					xml.WriteStartElement (null, "list-item", null);
+					
+					// Write the list items writing direction
+					xml.WriteStartAttribute (null, "dir", null);
+					if (Direction == Pango.Direction.Rtl)
+						xml.WriteString ("rtl");
+					else
+						xml.WriteString ("ltr");
+					xml.WriteEndAttribute ();
 				} else {
 					xml.WriteEndElement ();
 				}
@@ -413,7 +456,7 @@ namespace Tomboy
 
 			tag = new NoteTag ("note-title");
 			tag.Underline = Pango.Underline.Single;
-			tag.Foreground = "red";
+			tag.Foreground = "#204a87";
 			tag.Scale = Pango.Scale.XXLarge;
 			// FiXME: Hack around extra rewrite on open
 			tag.CanSerialize = false;
@@ -429,7 +472,7 @@ namespace Tomboy
 			tag = new NoteTag ("datetime");
 			tag.Scale = Pango.Scale.Small;
 			tag.Style = Pango.Style.Italic;
-			tag.Foreground = "grey";
+			tag.Foreground = "#888a85";
 			Add (tag);
 
 			// Font sizes
@@ -466,19 +509,19 @@ namespace Tomboy
 
 			tag = new NoteTag ("link:broken");
 			tag.Underline = Pango.Underline.Single;
-			tag.Foreground = "darkgrey";
+			tag.Foreground = "#555753";
 			tag.CanActivate = true;
 			Add (tag);
 
 			tag = new NoteTag ("link:internal");
 			tag.Underline = Pango.Underline.Single;
-			tag.Foreground = "red";
+			tag.Foreground = "#204a87";
 			tag.CanActivate = true;
 			Add (tag);
 
 			tag = new NoteTag ("link:url");
 			tag.Underline = Pango.Underline.Single;
-			tag.Foreground = "blue";
+			tag.Foreground = "#3465a4";
 			tag.CanActivate = true;
 			Add (tag);
 		}
@@ -526,18 +569,23 @@ namespace Tomboy
 			return false;
 		}
 
-		public DepthNoteTag GetDepthTag(int depth)
+		public DepthNoteTag GetDepthTag(int depth, Pango.Direction direction)
 		{
-			DepthNoteTag tag = Lookup ("depth:" + depth) as DepthNoteTag;
+			string name = "depth:" + depth + ":" + direction;
+			
+			DepthNoteTag tag = Lookup (name) as DepthNoteTag;
 
 			if (tag == null) {
-				tag = new DepthNoteTag (depth);
-				tag.Indent = -13;
-				tag.LeftMargin = (depth+1) * 25;
+				tag = new DepthNoteTag (depth, direction);
+				tag.Indent = -14;
+				
+				if (direction == Pango.Direction.Rtl)
+					tag.RightMargin = (depth+1) * 25;
+				else
+					tag.LeftMargin = (depth+1) * 25;
+				
 				tag.PixelsBelowLines = 4;
-				tag.Weight = Pango.Weight.Normal;
-				tag.Strikethrough = false;
-				tag.Style = Pango.Style.Normal;
+				tag.Scale = Pango.Scale.Medium;
 				Add (tag);
 			}
 
@@ -596,6 +644,6 @@ namespace Tomboy
 			}
 		}
 
-		public override event Gtk.TagChangedHandler TagChanged;
+		public new event Gtk.TagChangedHandler TagChanged;
 	}
 }
